library HmsFormProviderAddon;

uses
  Types,
  SysUtils,
  Classes,
  Windows,
  Variants,
  Forms,
  Dialogs,
  XPMan,
  HmsAddonsIntfTypes in 'HmsAddonsIntfTypes.pas',
  dlgHmsSetup in 'dlgHmsSetup.pas' {HmsSetupDialog},
  dlgHmsVideoItem in 'dlgHmsVideoItem.pas' {HmsVideoItemDialog},
  ParametersTools in 'ParametersTools.pas',
  dlgHmsFolder in 'dlgHmsFolder.pas' {HmsFolderDialog};

{$R *.res}

type
  TDispatchObject = class(TInterfacedObject, IDispatch)
  protected
    function GetTypeInfoCount(out Count: Integer): HResult; stdcall;
    function GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult; stdcall;
    function GetIDsOfNames(const IID: TGUID; Names: Pointer;
      NameCount, LocaleID: Integer; DispIDs: Pointer): HResult; stdcall;
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
      Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; stdcall;
  public
    destructor Destroy; override;
  end;

  THmsAddonList = class(TDispatchObject, IHmsAddonList, IHmsAddonTools, IHmsAddonInit)
  private
    FFormProvider: IHmsFormProvider;
  public
  { IHmsAddonInit }
    procedure SetApplication(const aApplication: IHmsApplication); safecall;
  { IHmsAddonTools }
    function Setup(aOwner: THandle; var aReload: LongBool): HResult; safecall;
    function Update(var aFlags: Integer; var aResult: OleVariant): HResult; safecall;
  { IHmsAddonList }
    function GetCount(var aCount: Integer): HResult; safecall;
    function GetAddonInfo(aIndex: Integer; var aClassID, aInterfaceID: TGUID;
       var aTitle, aDescription, aRequiredVersion, aCheckedOnVersion: OleVariant): HResult; safecall;
    function GetClassObject(const aClassID: TGUID; const aInterfaceID: TGUID; out aObject: OleVariant): HResult; safecall;
    function CanUnloadNow: HResult; safecall;
  end;

  THmsFormProvider = class(TDispatchObject, IHmsFormProvider)
  public
    function ShowModal(aOwner: THandle; const aFormClassName: OleVariant; var aParameters: OleVariant; var aModalResult: Integer): HResult; safecall;
  end;

{ THmsAddonList }

type
  THmsAddonInfo = record
    ClassID: TGUID;
    InterfaceID: TGUID;
    Title: string;
    Description: string;
    RequiredVersion: string;
    CheckedOnVersion: string;
  end;

const
  AddonInfoCount = 1;

  CLASS_HmsFormProvider = '{C5EC4D4D-3030-4598-9B9D-99D77987E233}';

  csAddonTitle = ' FormProvider';

  AddonInfoList: array[0..AddonInfoCount - 1] of THmsAddonInfo =
    ((ClassID : CLASS_HmsFormProvider;
      InterfaceID: IID_HmsFormProvider;
      Title : csAddonTitle;
      Description: '  -  ';
      RequiredVersion : '2.07';
      CheckedOnVersion: '2.07'));

{ TDispatchObject }

destructor TDispatchObject.Destroy;
begin
  inherited
end;

function TDispatchObject.GetTypeInfoCount(out Count: Integer): HResult;
begin
  Count := 0;
  Result := S_OK
end;

function TDispatchObject.GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult;
begin
  Result := E_INVALIDARG
end;

function TDispatchObject.GetIDsOfNames(const IID: TGUID; Names: Pointer;
                                          NameCount, LocaleID: Integer; DispIDs: Pointer): HResult;
begin
  Result := E_INVALIDARG
end;

function TDispatchObject.Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
      Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult;
begin
  Result := E_INVALIDARG
end;

{ THmsAddonList }

function THmsAddonList.GetCount(var aCount: Integer): HResult;
begin
  aCount := AddonInfoCount;
  Result := S_OK
end;

function THmsAddonList.GetAddonInfo(aIndex: Integer; var aClassID, aInterfaceID: TGUID;
       var aTitle, aDescription, aRequiredVersion, aCheckedOnVersion: OleVariant): HResult;
begin
  if (aIndex >= 0) and (aIndex < AddonInfoCount) then begin
    with AddonInfoList[aIndex] do begin
      aClassID := ClassID;
      aInterfaceID := InterfaceID;
      aTitle := Title;
      aDescription := Description;
      aRequiredVersion := RequiredVersion;
      aCheckedOnVersion := CheckedOnVersion
    end;
    Result := S_OK
  end else
    Result := E_UNEXPECTED
end;

function THmsAddonList.GetClassObject(const aClassID: TGUID; const aInterfaceID: TGUID; out aObject: OleVariant): HResult;
begin
  Result := E_NOINTERFACE;
  if IsEqualGUID(aInterfaceID, IHmsFormProvider) then begin
    if FFormProvider = nil then
      FFormProvider := THmsFormProvider.Create;
    aObject := FFormProvider;
    Result := S_OK
  end
end;

function THmsAddonList.CanUnloadNow: HResult;
begin
  FFormProvider := nil;
  Result := S_OK
end;

function THmsAddonList.Setup(aOwner: THandle; var aReload: LongBool): HResult;
begin
  aReload := MessageBox(aOwner, '  ?', '  ', MB_ICONQUESTION + MB_YESNO) = IDYES;
  Result := S_OK
end;

function THmsAddonList.Update(var aFlags: Integer; var aResult: OleVariant): HResult;
begin
  aFlags := ufIsInfoMessage;
  aResult := '  ';
  Result := S_OK
end;

procedure THmsAddonList.SetApplication(const aApplication: IHmsApplication);
begin
  HmsApplication := aApplication
end;

{ THmsFormProvider }

function THmsFormProvider.ShowModal(aOwner: THandle; const aFormClassName: OleVariant; var aParameters: OleVariant; var aModalResult: Integer): HResult;
var
  sFormClassName: string;
begin
  Application.Handle := aOwner;
  try
    Result := S_OK;
    sFormClassName := VarToStr(aFormClassName);
    ShowMessage(Format('%s ('#13#10'%s'#13#10')', [sFormClassName, ParametersText(aParameters, '     ')]));
    if SameText(sFormClassName, 'THmsSetupDialog') then
      aModalResult := ExecuteSetupDialog(aParameters)
    else if SameText(sFormClassName, 'THmsVideoItemDialog') then
      aModalResult := ExecuteVideoItemDialog(aParameters)
    else if SameText(sFormClassName, 'THmsFolderDialog') then
      aModalResult := ExecuteFolderDialog(aParameters)
    else
      Result := E_NOTIMPL;
    if Result = S_OK then
      ShowMessage(Format('%s ('#13#10'%s'#13#10')', [sFormClassName, ParametersText(aParameters, '     ')]));
  finally
    Application.Handle := 0
  end
end;

function HmsGetAddonList(out aAddonList: IHmsAddonList): HResult; stdcall;
begin
  aAddonList := THmsAddonList.Create;
  Result := S_OK
end;

exports
  HmsGetAddonList;

end.
