unit ParametersTools;

interface

  function GetParameter(const aParameters: OleVariant; const aParameterName: string): OleVariant;

  function GetInterfaceParameter(const aParameters: OleVariant;
       const aParameterName: string; const aInterfaceID: TGUID; out aObject): LongBool;

  function InterfaceFromVariant(const aValue: OleVariant; const aInterfaceID: TGUID; out aObject): LongBool;

  function ParametersText(const aParameters: Variant; const aOffset: string = ''): string;  

  procedure SetParameter(var aParameters: OleVariant; const aParameterName: string; const aParameterValue: OleVariant);

implementation

uses
  HmsAddonsIntfTypes, SysUtils, Variants;

function GetParameter(const aParameters: OleVariant; const aParameterName: string): OleVariant;
var
  i, iCount: Integer;
  ParameterName: Variant;
begin
  Result := '';
  if VarIsArray(aParameters) then begin
    i := 0; iCount := VarArrayHighBound(aParameters, 1) + 1;
    while i < iCount - 1 do begin
      ParameterName := aParameters[i];
      if VarIsStr(ParameterName) and SameText(ParameterName, aParameterName) then begin
        Result := aParameters[i + 1]; Break
      end;
      Inc(i, 2)
    end
  end
end;

function GetInterfaceParameter(const aParameters: OleVariant; const aParameterName: string; const aInterfaceID: TGUID; out aObject): LongBool;
begin
  Result := InterfaceFromVariant(GetParameter(aParameters, aParameterName), aInterfaceID, aObject)
end;

procedure SetParameter(var aParameters: OleVariant; const aParameterName: string; const aParameterValue: OleVariant);
var
  i, iCount: Integer;
  ParameterName: Variant;
begin
  if VarIsArray(aParameters) then begin
    i := 0; iCount := VarArrayHighBound(aParameters, 1) + 1;
    while i < iCount - 1 do begin
      ParameterName := aParameters[i];
      if VarIsStr(ParameterName) and SameText(ParameterName, aParameterName) then begin
        aParameters[i + 1] := aParameterValue;
        Break
      end;
      Inc(i, 2)
    end
  end
end;

function GetIntertfaceName(const aInterface: IUnknown): string;
var
  HmsAddonMediaItem: IHmsAddonMediaItem;
  HmsAddonMediaItemListNavigator: IHmsAddonMediaItemListNavigator;
begin
  if aInterface <> nil then begin
    if aInterface.QueryInterface(IHmsAddonMediaItem, HmsAddonMediaItem) = S_OK then
      Result := 'IHmsAddonMediaItem'
    else if aInterface.QueryInterface(IHmsAddonMediaItemListNavigator, HmsAddonMediaItemListNavigator) = S_OK then
      Result := 'IHmsAddonMediaItemListNavigator'
    else
      Result := 'IUnknown'
  end else
    Result := 'nil'    
end;

function VarIsInterface(const aValue: OleVariant): LongBool; inline;
begin
  case VarType(aValue) of
    varDispatch, varDispatch + varByRef, varUnknown, varUnknown + varByRef: Result := True;
  else
    Result := False
  end    
end;

function SafeVarToStr(const aValue: OleVariant): string;
begin
  if VarIsInterface(aValue) then
    Result := GetIntertfaceName(aValue)
  else try
    Result := VarToStr(aValue)
  except
    Result := 'Unknown'
  end
end;

function ParametersText(const aParameters: Variant; const aOffset: string = ''): string;
var
  i, iCount: Integer;
  ParameterName, ParameterValue: Variant;
begin
  Result := '';
  if VarIsArray(aParameters) then begin
    i := 0; iCount := VarArrayHighBound(aParameters, 1) + 1;
    while i < iCount - 1 do begin
      ParameterName := aParameters[i];
      ParameterValue := aParameters[i + 1];
      if VarIsStr(ParameterName) then begin
        if Result <> '' then Result := Result + #13#10;
        Result := Result + aOffset + VarToStr(ParameterName) + ': ';
        if VarIsArray(ParameterValue) then
          Result := Result + '[' + #13#10 + ParametersText(ParameterValue, aOffset + '    ') + ']'
        else
          Result := Result + SafeVarToStr(ParameterValue)
      end;
      Inc(i, 2)
    end
  end
end;

function InterfaceFromVariant(const aValue: OleVariant; const aInterfaceID: TGUID; out aObject): LongBool;
var
  Unknown: IUnknown;
begin
  Result := False;
  if VarIsInterface(aValue) then begin
    Unknown := aValue;
    if Assigned(Unknown) then
      Result := Unknown.QueryInterface(aInterfaceID, aObject) = S_OK
  end
end;

end.
