unit dlgHmsFolder;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, HmsAddonsIntfTypes;

type
  THmsFolderDialog = class(TForm)
    gbSetupDialog: TGroupBox;
    lblItemTitle: TLabel;
    lblItemPath: TLabel;
    edtItemTitle: TEdit;
    edtItemPath: TEdit;
    btnFirst: TButton;
    btnPrior: TButton;
    btnNext: TButton;
    btnLast: TButton;
    btnOK: TButton;
    btnCancel: TButton;
    btnScripterTest: TButton;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btnFirstClick(Sender: TObject);
    procedure btnScripterTestClick(Sender: TObject);
  private
    { Private declarations }
    FMediaItem: IHmsAddonMediaItem;
    FNavigator: IHmsAddonMediaItemListNavigator;

    procedure EnableButtons;

    procedure GetInfo(const aMediaItem: IHmsAddonMediaItem);
    procedure SetInfo(const aMediaItem: IHmsAddonMediaItem);
  public
    { Public declarations }
    procedure Init(const aParameters: Variant);
  end;

var
  HmsFolderDialog: THmsFolderDialog;

  function ExecuteFolderDialog(var aParameters: OleVariant): Integer;

implementation

{$R *.dfm}

uses
  ParametersTools;

function ExecuteFolderDialog(var aParameters: OleVariant): Integer;
begin
  with THmsFolderDialog.Create(nil) do try
    Init(aParameters);
    Result := ShowModal
  finally
    Free
  end
end;

procedure THmsFolderDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if (ModalResult = mrOK) and (FMediaItem <> nil) then
    SetInfo(FMediaItem)
end;

procedure THmsFolderDialog.Init(const aParameters: Variant);
var
  sCaption: string;
begin
  sCaption := GetParameter(aParameters, 'Caption');
  if sCaption <> '' then Caption := sCaption;
  GetInterfaceParameter(aParameters, 'MediaItem', IHmsAddonMediaItem, FMediaItem);
  GetInterfaceParameter(aParameters, 'Navigator', IHmsAddonMediaItemListNavigator, FNavigator);
  GetInfo(FMediaItem)
end;

procedure THmsFolderDialog.GetInfo(const aMediaItem: IHmsAddonMediaItem);
begin
  FMediaItem := aMediaItem;
  if Assigned(FMediaItem) then begin
    edtItemTitle.Text := VarToStr(FMediaItem['mpTitle']);
    edtItemPath.Text := VarToStr(FMediaItem['mpFilePath']);
  end else begin
    edtItemTitle.Text := '';
    edtItemPath.Text := '';
  end;
  EnableButtons
end;

procedure THmsFolderDialog.SetInfo(const aMediaItem: IHmsAddonMediaItem);
begin
  if Assigned(aMediaItem) then
    aMediaItem['mpTitle'] := edtItemTitle.Text
end;

procedure THmsFolderDialog.btnFirstClick(Sender: TObject);
begin
  if FNavigator <> nil then begin
    SetInfo(FMediaItem);
    case TComponent(Sender).Tag of
      1: FNavigator.GotoBOF;
      2: FNavigator.GotoPrev;
      3: FNavigator.GotoNext;
      4: FNavigator.GotoEOF;
    end;
    InterfaceFromVariant(FNavigator.CurrentMediaItem, IHmsAddonMediaItem, FMediaItem);
    GetInfo(FMediaItem);
  end
end;

procedure THmsFolderDialog.btnScripterTestClick(Sender: TObject);
var
  ovHmsScripter: OleVariant;
  HmsScripter: IHmsScripter;
  ErrorMessage, InParameters, OutParameters: OleVariant;
  iErrorLine, iErrorChar: Integer;
begin
  if Assigned(HmsApplication) then begin
    if HmsApplication.GetInterface(IHmsScripter, ovHmsScripter) = S_OK then begin
      if InterfaceFromVariant(ovHmsScripter, IHmsScripter, HmsScripter) then begin
        if HmsScripter.CreateScript(Null, Ord(smProcessMedia)) then begin
          if HmsScripter.CompileScript('PascalScript',
               'var'#13#10 +
               '  TestVarIn, TestVarOut: string;'#13#10 +
               'begin'#13#10 +
               '  ShowMessage(CurrentMediaItem.ItemTitle + '#39': '#39' + TestVarIn);'#13#10 +
               '  TestVarOut := '#39'12345'#39';'#13#10 +
               'end.', ErrorMessage, iErrorLine, iErrorChar) then begin
            InParameters := VarArrayOf(['CurrentMediaItem', FMediaItem, 'TestVarIn', '54321']);
            OutParameters := VarArrayOf(['TestVarOut', Null]);
            if HmsScripter.ExecuteScript(InParameters, OutParameters) then begin
              ShowMessage(OutParameters[1]);
            end
          end else
            ShowMessage(Format('[%d:%d]: %s', [iErrorLine, iErrorChar, VarToStr(ErrorMessage)]))
        end
      end
    end
  end
end;

procedure THmsFolderDialog.EnableButtons;
begin
  if FNavigator <> nil then begin
    btnFirst.Enabled := not FNavigator.IsBOF;
    btnLast.Enabled := not FNavigator.IsEOF;
    btnPrior.Enabled := btnFirst.Enabled;
    btnNext.Enabled := btnLast.Enabled;
  end else begin
    btnFirst.Enabled := False;
    btnPrior.Enabled := False;
    btnNext.Enabled := False;
    btnLast.Enabled := False;
  end
end;

end.
