unit dlgHmsSetup;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, XMLDoc, XMLIntf;

type
  THmsSetupDialog = class(TForm)
    gbSetupDialog: TGroupBox;
    btnOK: TButton;
    btnCancel: TButton;
    pnlButtons: TPanel;
    btnAdd: TButton;
    btnEdit: TButton;
    btnDelete: TButton;
    lbDirectoryList: TListBox;
    procedure btnAddClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private declarations }
    FConfigFile: IXMLDocument;
    FMediaWatchFoldersNode: IXMLNode;
    FSaveDirectoryList: string;
    procedure LoadDirectoryList;
    procedure SaveDirectoryList;
  public
    { Public declarations }
    procedure Init(const aParameters: Variant);
  end;

var
  HmsSetupDialog: THmsSetupDialog;

  function ExecuteSetupDialog(var aParameters: OleVariant): Integer;

implementation

{$R *.dfm}

uses
  FileCtrl, ParametersTools;

function YesNo(const Msg : string; DlgType : TMsgDlgType = mtConfirmation) : Boolean;
begin
  Result := MessageDlg(Msg, DlgType, [mbYes, mbNo], 0) = mrYes
end;

function ExecuteSetupDialog(var aParameters: OleVariant): Integer;
begin
  with THmsSetupDialog.Create(nil) do try
    Init(aParameters);
    Result := ShowModal;
    if (Result = mrOK) and YesNo('   - ?') then
      SetParameter(aParameters, 'ScanFolders', True);
  finally
    Free
  end
end;

procedure THmsSetupDialog.LoadDirectoryList;
var
  i: Integer;
begin
  lbDirectoryList.Items.BeginUpdate;
  try
    lbDirectoryList.Items.Clear;
    if FConfigFile.DocumentElement <> nil then begin
      FMediaWatchFoldersNode := FConfigFile.DocumentElement.ChildNodes.FindNode('MediaWatchFolders');
      if FMediaWatchFoldersNode <> nil then
        for i := 0 to FMediaWatchFoldersNode.ChildNodes.Count - 1 do
          lbDirectoryList.Items.Add(FMediaWatchFoldersNode.ChildNodes[i].ChildValues['Folder']);
    end
  finally
    lbDirectoryList.Items.EndUpdate
  end;
  FSaveDirectoryList := lbDirectoryList.Items.Text
end;

procedure THmsSetupDialog.SaveDirectoryList;
var
  i: Integer;
begin
  if FMediaWatchFoldersNode <> nil then begin
    FMediaWatchFoldersNode.ChildNodes.Clear;
    for i := 0 to lbDirectoryList.Items.Count - 1 do
      FMediaWatchFoldersNode.AddChild('Folder').ChildValues['Folder'] := lbDirectoryList.Items[i];
    FConfigFile.SaveToFile(FConfigFile.FileName)
  end
end;

procedure THmsSetupDialog.Init(const aParameters: Variant);
var
  sConfigFileName: string;
begin
  sConfigFileName := GetParameter(aParameters, 'ConfigFileName');
  if sConfigFileName <> '' then begin
    FConfigFile := LoadXMLDocument(sConfigFileName);
    FConfigFile.Options := [doNodeAutoCreate, doNodeAutoIndent, doAutoPrefix];
    FConfigFile.NodeIndentStr := '  ';
    LoadDirectoryList;
  end
end;

procedure THmsSetupDialog.btnAddClick(Sender: TObject);
var
  i: Integer;
  sDirectory: string;
begin
  if SelectDirectory(' ', '', sDirectory) then begin
    sDirectory := IncludeTrailingPathDelimiter(sDirectory);
    i := lbDirectoryList.Items.IndexOf(sDirectory);
    if i = -1 then
      i := lbDirectoryList.Items.Add(sDirectory);
    lbDirectoryList.ItemIndex := i
  end
end;

procedure THmsSetupDialog.btnDeleteClick(Sender: TObject);
begin
  if lbDirectoryList.ItemIndex <> -1 then
    if YesNo(Format('  "%s"   ?', [lbDirectoryList.Items[lbDirectoryList.ItemIndex]])) then
      lbDirectoryList.Items.Delete(lbDirectoryList.ItemIndex)
end;

procedure THmsSetupDialog.btnEditClick(Sender: TObject);
var
  sDirectory: string;
begin
  if lbDirectoryList.ItemIndex <> -1 then begin
    sDirectory := lbDirectoryList.Items[lbDirectoryList.ItemIndex];
    if SelectDirectory(' ', '', sDirectory) then begin
      sDirectory := IncludeTrailingPathDelimiter(sDirectory);
      lbDirectoryList.Items[lbDirectoryList.ItemIndex] := sDirectory
    end  
  end
end;

procedure THmsSetupDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if ModalResult = mrOK then begin
    if FSaveDirectoryList <> lbDirectoryList.Items.Text then
      SaveDirectoryList
    else
      ModalResult := mrCancel
  end
end;

procedure THmsSetupDialog.FormDestroy(Sender: TObject);
begin
  FConfigFile := nil
end;

end.
