unit dlgHmsVideoItem;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, HmsAddonsIntfTypes, ExtCtrls;

type
  THmsVideoItemDialog = class(TForm)
    gbSetupDialog: TGroupBox;
    btnOK: TButton;
    btnCancel: TButton;
    lblItemTitle: TLabel;
    edtItemTitle: TEdit;
    lblItemPath: TLabel;
    edtItemPath: TEdit;
    btnFirst: TButton;
    btnPrior: TButton;
    btnNext: TButton;
    btnLast: TButton;
    Image1: TImage;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btnFirstClick(Sender: TObject);
  private
    { Private declarations }
    FHmsThumbnailsManager: IHmsThumbnailsManager;
    FMediaItem: IHmsAddonMediaItem;
    FNavigator: IHmsAddonMediaItemListNavigator;

    procedure EnableButtons;

    procedure GetThumbnail;

    procedure GetInfo(const aMediaItem: IHmsAddonMediaItem);
    procedure SetInfo(const aMediaItem: IHmsAddonMediaItem);
  public
    { Public declarations }
    procedure Init(const aParameters: Variant);
  end;

var
  HmsVideoItemDialog: THmsVideoItemDialog;

  function ExecuteVideoItemDialog(var aParameters: OleVariant): Integer;

implementation

{$R *.dfm}

uses
  ParametersTools, PngImage, GifImg, Jpeg;

function ExecuteVideoItemDialog(var aParameters: OleVariant): Integer;
begin
  with THmsVideoItemDialog.Create(nil) do try
    Init(aParameters);
    Result := ShowModal
  finally
    Free
  end
end;

procedure THmsVideoItemDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if (ModalResult = mrOK) and (FMediaItem <> nil) then
    SetInfo(FMediaItem)
end;

procedure THmsVideoItemDialog.Init(const aParameters: Variant);
var
  ovHmsThumbnailsManager: OleVariant;
begin
  GetInterfaceParameter(aParameters, 'MediaItem', IHmsAddonMediaItem, FMediaItem);
  GetInterfaceParameter(aParameters, 'Navigator', IHmsAddonMediaItemListNavigator, FNavigator);
  if Assigned(HmsApplication) then
    if HmsApplication.GetInterface(IHmsThumbnailsManager, ovHmsThumbnailsManager) = S_OK then
      InterfaceFromVariant(ovHmsThumbnailsManager, IHmsThumbnailsManager, FHmsThumbnailsManager);
  GetInfo(FMediaItem)
end;

procedure THmsVideoItemDialog.GetInfo(const aMediaItem: IHmsAddonMediaItem);
begin
  FMediaItem := aMediaItem;
  if Assigned(FMediaItem) then begin
    edtItemTitle.Text := VarToStr(FMediaItem.Properties['mpTitle']);
    edtItemPath.Text := VarToStr(FMediaItem.Properties['mpFilePath']);
    GetThumbnail;
  end else begin
    edtItemTitle.Text := '';
    edtItemPath.Text := '';
  end;
  EnableButtons
end;

procedure THmsVideoItemDialog.SetInfo(const aMediaItem: IHmsAddonMediaItem);
begin
  if Assigned(aMediaItem) then
    aMediaItem.Properties['mpTitle'] := edtItemTitle.Text
end;

procedure THmsVideoItemDialog.btnFirstClick(Sender: TObject);
begin
  if FNavigator <> nil then begin
    SetInfo(FMediaItem);
    case TComponent(Sender).Tag of
      1: FNavigator.GotoBOF;
      2: FNavigator.GotoPrev;
      3: FNavigator.GotoNext;
      4: FNavigator.GotoEOF;
    end;
    InterfaceFromVariant(FNavigator.CurrentMediaItem, IHmsAddonMediaItem, FMediaItem);
    GetInfo(FMediaItem);
  end
end;

function GetGraphicClass(const aThumbnailFormat: string): TGraphicClass;
begin
  if SameText(aThumbnailFormat, 'image/png') then
    Result := TPngImage
  else if SameText(aThumbnailFormat, 'image/gif') then
    Result := TGifImage
  else
    Result := TJpegImage
end;

procedure THmsVideoItemDialog.GetThumbnail;
var
  Graphic: TGraphic;
  sThumbnailKey: string;
  ThumbnailFormat: OleVariant;
  iThumbnailWidth, iThumbnailHeight: Integer;
  ThumbnailStream: TMemoryStream;
begin
  if Assigned(FHmsThumbnailsManager) then begin
    sThumbnailKey := VarToStr(FMediaItem['ThumbnailKey']);
    if sThumbnailKey <> '' then begin
      ThumbnailFormat := Null;
      if FHmsThumbnailsManager.GetThumbnailInfo(sThumbnailKey, ThumbnailFormat,
                                                iThumbnailWidth, iThumbnailHeight) then begin
        ThumbnailStream := TMemoryStream.Create;
        try
          if FHmsThumbnailsManager.GetThumbnail(sThumbnailKey,
                                 TStreamAdapter.Create(ThumbnailStream)) then begin
            Graphic := GetGraphicClass(VarToStr(ThumbnailFormat)).Create;
            try
              ThumbnailStream.Position := 0;
              Graphic.LoadFromStream(ThumbnailStream);
              Image1.Picture.Graphic := Graphic
            finally
              Graphic.Free
            end
          end
        finally
          ThumbnailStream.Free
        end
      end
    end
  end
end;

procedure THmsVideoItemDialog.EnableButtons;
begin
  if FNavigator <> nil then begin
    btnFirst.Enabled := not FNavigator.IsBOF;
    btnLast.Enabled := not FNavigator.IsEOF;
    btnPrior.Enabled := btnFirst.Enabled;
    btnNext.Enabled := btnLast.Enabled;
  end else begin
    btnFirst.Enabled := False;
    btnPrior.Enabled := False;
    btnNext.Enabled := False;
    btnLast.Enabled := False;
  end
end;

end.
